import json
import segment_source.domain_pb2_grpc as svc
import segment_source.domain_pb2 as service
from segment_source.stats import Stats
from segment_source.lsp import SourceLogger
import grpc

_TIMEOUT = 10


INTERNAL_ERROR = "internal"
AUTHENTICATION_ERROR = "authentication"
PERMISSIONS_ERROR = "permissions"
VALIDATION_ERROR = "validation"
PARTNER_ERROR = "partner"
RATE_LIMIT_ERROR = "rate_limit"

channel = grpc.insecure_channel('localhost:4000')
client = svc.SourceStub(channel)
stats = Stats()
lsp = SourceLogger(client, "", "")


##
# Helpers
##

def _byteify(d):
    json_str = json.dumps(d)
    bytes_ = json_str.encode()

    return bytes_

##
# API
##


def set(collection, id_, properties):
    _properties = _byteify(properties)
    request = service.SetRequest(
        collection=collection, id=id_, properties=_properties)
    stats.incr("set")
    client.Set(request, _TIMEOUT)


def track(context=None, integrations=None, properties=None, anonymous_id=None,
          user_id=None, event=None):
    pass


def identify(context=None, integrations=None, traits=None, anonymous_id=None,
             user_id=None):
    pass


def group(context=None, integrations=None, traits=None, anonymous_id=None,
          user_id=None, event=None):
    pass


def keep_alive():
    stats.incr("keep_alive")
    client.KeepAlive(service.Empty(), _TIMEOUT)


def get_context(allow_failed=False):
    request = service.GetContextRequest(allowFailed=allow_failed)
    return client.GetContext(request, _TIMEOUT).data


def get_context_into_file(allow_failed=False):
    request = service.GetContextIntoFileRequest(allowFailed=allow_failed)
    return client.GetContextIntoFile(request, _TIMEOUT).filename


def store_context(payload):
    request = service.StoreContextRequest(payload=payload)
    client.StoreContext(request, _TIMEOUT)


def store_context_from_file(filename):
    request = service.StoreContextFromFileRequest(filename=filename)
    client.StoreContextFromFile(request, _TIMEOUT)


def report_error(message, category=None, collection=None):
    request = service.ReportErrorRequest(
        message=message, category=category, collection=collection)
    client.ReportError(request, _TIMEOUT)


def report_warning(message, category=None, collection=None):
    request = service.ReportWarningRequest(
        message=message, category=category, collection=collection)
    client.ReportWarning(request, _TIMEOUT)


def datadog_metric_incr(name, value=1, tags=None):
    request = service.StatsRequest(name=name, value=value, tags=tags)
    client.StatsIncrement(request, _TIMEOUT)


def datadog_metric_observe(name, value, tags=None):
    request = service.StatsRequest(name=name, value=value, tags=tags)
    client.StatsHistogram(request, _TIMEOUT)


def datadog_metric_gauge(name, value, tags=None):
    request = service.StatsRequest(name=name, value=value, tags=tags)
    client.StatsGauge(request, _TIMEOUT)

def s3_metric_incr(name, value=1, tags=None):
    request = service.StatsRequest(name=name, value=value, tags=tags, backend='s3')
    client.StatsIncrement(request, _TIMEOUT)


def s3_metric_observe(name, value, tags=None):
    request = service.StatsRequest(name=name, value=value, tags=tags, backend='s3')
    client.StatsHistogram(request, _TIMEOUT)


def s3_metric_guage(name, value, tags=None):
    request = service.StatsRequest(name=name, value=value, tags=tags, backend='s3')
    client.StatsGauge(request, _TIMEOUT)


def log():
    return stats.log()


def last_touched():
    return stats.last_touched
